/**
 * @file
 * VuoList_VuoMidiController C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoMidiController_H
#define VuoList_VuoMidiController_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoMidiController VuoList_VuoMidiController
 * A list of @ref VuoMidiController elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoMidiController_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoMidiController elements.
 */
typedef const struct VuoList_VuoMidiController_struct { void *l; } * VuoList_VuoMidiController;
/// @}
#endif

/**
 * Creates a new list of @ref VuoMidiController elements.
 */
VuoList_VuoMidiController VuoListCreate_VuoMidiController(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoMidiController VuoListCopy_VuoMidiController(const VuoList_VuoMidiController list);

/**
 * Returns the @ref VuoMidiController at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoMidiController VuoListGetValue_VuoMidiController(const VuoList_VuoMidiController list, const unsigned long index);

/**
 * Changes the @ref VuoMidiController at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoMidiController(const VuoList_VuoMidiController list, const VuoMidiController value, const unsigned long index);

/**
 * Inserts the `VuoMidiController` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoMidiController(const VuoList_VuoMidiController list, const VuoMidiController value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoMidiController(VuoList_VuoMidiController list, const VuoMidiController value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoMidiController(VuoList_VuoMidiController list, const VuoMidiController value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoMidiController(VuoList_VuoMidiController list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoMidiController(VuoList_VuoMidiController list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoMidiController(VuoList_VuoMidiController list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoMidiController(VuoList_VuoMidiController list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoMidiController(VuoList_VuoMidiController list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoMidiController(VuoList_VuoMidiController list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoMidiController(VuoList_VuoMidiController list);

/**
 * Removes the `VuoMidiController` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoMidiController(VuoList_VuoMidiController list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoMidiController(const VuoList_VuoMidiController list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoMidiController VuoList_VuoMidiController_valueFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoMidiController_jsonFromValue(const VuoList_VuoMidiController value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoMidiController_summaryFromValue(const VuoList_VuoMidiController value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoMidiController VuoList_VuoMidiController_valueFromString(const char *str);
char * VuoList_VuoMidiController_stringFromValue(const VuoList_VuoMidiController value);
/// @}

/**
 * @}
 */

#endif
